#!/usr/bin/python
# encoding=utf-8

"""
@Author  :  Lijiawei
@Date    :  2021/12/8 5:59 下午
@Desc    :  plugins line.
"""

import traceback
import importlib

from airtest.utils.logger import get_logger

LOGGING = get_logger(__name__)


class TestPlugin:
    @staticmethod
    def show_me():
        print('plugin_name')


def plugin(plugin_name: str, sep=':'):
    """

    :param plugin_name:
    :param sep:
    :return:
    """
    m, _, c = plugin_name.partition(sep)
    mod = importlib.import_module(m)
    cls = getattr(mod, c)
    return cls()


def init_plugin_modules(plugins):
    """
    动态导入插件
    :param plugins:
    :return:
    """
    if not plugins:
        return
    for plugin_name in plugins:
        LOGGING.debug("try loading plugin: %s" % plugin_name)
        try:
            __import__(plugin_name)
        except ImportError:
            LOGGING.error(traceback.format_exc())


if __name__ == '__main__':
    pg = plugin('_load:TestPlugin')
    pg.show_me()



#!/usr/bin/python
# encoding=utf-8

"""
@Author  :  Lijiawei
@Date    :  11/02/2021 3:36 PM
@Desc    :  utx selenium report plugin
"""

import os
import utx.core.css.report as report

LOGDIR = "log"

old_trans_screen = report.LogToHtml._translate_screen
old_trans_desc = report.LogToHtml._translate_desc
old_trans_code = report.LogToHtml._translate_code

screen_func = ["find_element_by_xpath", "find_element_by_id", "find_element_by_name", "assert_exist",
               "back", "forward", "switch_to_new_tab", "switch_to_previous_tab", "get",
               "click", "send_keys"]

second_screen_func = ["click", "send_keys"]
other_func = []


def new_trans_screen(self, step, code):
    trans = old_trans_screen(self, step, code)
    if "name" in step['data'] and step['data']["name"] in screen_func:
        screen = {
            "src": None,
            "rect": [],
            "pos": [],
            "vector": [],
            "confidence": None,
        }

        src = ""
        if step["data"]["name"] in second_screen_func:
            res = step["data"]['ret']
            src = res["screen"]
            if "pos" in res:
                screen["pos"] = res["pos"]

        for item in step["__children__"]:
            if item["data"]["name"] == "_gen_screen_log":
                res = item["data"]['ret']
                src = res["screen"]
                if "pos" in res:
                    screen["pos"] = res["pos"]
                break

        if self.export_dir and src:
            src = os.path.join(LOGDIR, src)
        screen["src"] = src

        return screen
    else:
        return trans


def new_translate_desc(self, step, code):
    trans = old_trans_desc(self, step, code)
    if "name" in step['data'] and (step['data']["name"] in screen_func or step["data"]["name"] in other_func):
        name = step["data"]["name"]
        args = {i["key"]: i["value"] for i in code["args"]}
        desc = {
            "find_element_by_xpath": lambda: u"Find element by xpath: %s" % args.get("xpath"),
            "find_element_by_id": lambda: u"Find element by id: %s" % args.get("id"),
            "find_element_by_name": lambda: u"Find element by name: %s" % args.get("name"),
            "assert_exist": u"Assert element exists.",
            "click": u"Click the element that been found.",
            "send_keys": u"Send some text and keyboard event to the element that been found.",
            "get": lambda: u"Get the web address: %s" % args.get("address"),
            "switch_to_last_window": u"Switch to the last tab.",
            "switch_to_latest_window": u"Switch to the new tab.",
            "back": u"Back to the last page.",
            "forward": u"Forward to the next page.",
            "snapshot": u"Snopshot current page."
        }

        desc_zh = {
            "find_element_by_xpath": lambda: u"寻找指定页面元素: \"%s\"" % args.get("xpath"),
            "find_element_by_id": lambda: u"寻找指定页面元素: \"%s\"" % args.get("id"),
            "find_element_by_name": lambda: u"寻找指定页面元素: \"%s\"" % args.get("name"),
            "assert_exist": u"断言页面元素存在.",
            "click": u"点击找到的页面元素.",
            "send_keys": u"传送文本\"%s\"到选中文本框." % (args.get("text", "")),
            "get": lambda: u"访问网络地址: %s" % args.get("address"),
            "switch_to_last_window": u"切换到上一个标签页.",
            "switch_to_latest_window": u"切换到最新标签页.",
            "back": u"后退到上一个页面.",
            "forward": u"前进到下一个页面.",
            "snapshot": u"截取当前页面."
        }

        if self.lang == 'zh':
            desc = desc_zh
        ret = desc.get(name)
        if callable(ret):
            ret = ret()
        return ret
    else:
        return trans


def new_translate_code(self, step):
    trans = old_trans_code(self, step)
    if trans:
        for idx, item in enumerate(trans["args"]):
            if item["key"] == "self":
                trans["args"].pop(idx)
    return trans


report.LogToHtml._translate_screen = new_trans_screen
report.LogToHtml._translate_desc = new_translate_desc
report.LogToHtml._translate_code = new_translate_code



#!/usr/bin/python
# encoding=utf-8

"""
@Author  :  Lijiawei
@Date    :  9/14/2021 3:36 PM
@Desc    :  Hook line.
"""
import logging
import random

import allure
import pytest

from logging import getLogger
from airtest.core.api import *
from airtest.core.helper import device_platform, ST
from selenium.webdriver.chrome.options import Options
from selenium import webdriver
from tenacity import Retrying, wait_fixed, stop_after_attempt
from wda import WDAError
from utx.core.utils.tools import str_to_bool, check_port, decryption, plat

logger = getLogger(__name__)


def app_info(cli_platform, cli_device, cli_wda, cli_init, ini_platform, ini_device, ini_wda, ini_init):
    """

    :param cli_platform:
    :param cli_device:
    :param cli_wda:
    :param cli_init:
    :param ini_platform:
    :param ini_device:
    :param ini_wda:
    :param ini_init:
    :return: iOS:///127.0.0.1:8100
    """
    if cli_init is None:
        is_init = str_to_bool(ini_init)
    else:
        is_init = str_to_bool(cli_init)
    if cli_wda is None:
        WebDriverAgent = ini_wda
    else:
        WebDriverAgent = cli_wda

    if cli_platform is None:
        platform = ini_platform.lower()
        if platform in 'android':
            device_idx = ini_device
            device_uri = 'Android:///{}?cap_method=JAVACAP&&ori_method=ADBORI&&touch_method=MAXTOUCH'.format(device_idx)
            return device_uri.split(','), device_idx, is_init
        elif platform in 'ios':
            proxy = str(random.randint(50000, 60000) + 1)
            check_port(port=proxy)
            os.system('{} wdaproxy -B {} --port {} &'.format(decryption(b'dGlkZXZpY2U='), WebDriverAgent, proxy))
            device_idx = '127.0.0.1:{}'.format(proxy)
            device_uri = 'iOS:///127.0.0.1:{}'.format(proxy)
            return device_uri.split(','), device_idx, is_init

    elif cli_platform is not None:
        if cli_platform.lower() in 'android':
            device_idx = cli_device
            device_uri = 'Android:///{}?cap_method=JAVACAP&&ori_method=ADBORI&&touch_method=MAXTOUCH'.format(device_idx)
            return device_uri.split(','), device_idx, is_init
        elif cli_platform.lower() in 'ios':
            proxy = str(random.randint(50000, 60000) + 1)
            check_port(port=proxy)
            os.system('{} wdaproxy -B {} --port {} &'.format(decryption(b'dGlkZXZpY2U='), WebDriverAgent, proxy))
            device_idx = '127.0.0.1:{}'.format(proxy)
            device_uri = 'iOS:///127.0.0.1:{}'.format(proxy)
            return device_uri.split(','), device_idx, is_init


def my_before_sleep(retry_state):
    """

    :param retry_state:
    :return:
    """
    if retry_state.attempt_number < 1:
        loglevel = logging.INFO
    else:
        loglevel = logging.WARNING

    logging.log(
        loglevel,
        'Retrying %s: attempt %s ended with: %s',
        retry_state.fn,
        retry_state.attempt_number,
        retry_state.outcome,
    )


@allure.step("Try to link the device！")
def my_retry_connect(uri=None, whether_retry=True, sleeps=10, max_attempts=3):
    """

    :param uri:
    :param whether_retry:
    :param sleeps:
    :param max_attempts:
    :return:
    """
    if not whether_retry:
        max_attempts = 1

    r = Retrying(wait=wait_fixed(sleeps), stop=stop_after_attempt(max_attempts), before_sleep=my_before_sleep,
                 reraise=True)
    try:
        return r(connect_device, uri)
    except Exception as e:
        if isinstance(e, (WDAError,)):
            logger.info("Can't connect iphone, please check device or wda state!")
        logger.info("Try connect device {} 3 times per wait 10 sec failed.".format(uri))
        raise e
    finally:
        logger.info("Retry connect statistics: {}".format(str(r.statistics)))


@allure.step("Switch to current device！")
def ensure_current_device(device_idx):
    """

    :param device_idx:
    :return:
    """
    idx = device_idx
    try:
        if device().uuid != idx:
            set_current(idx)
    except IndexError:
        if device().uuid != 'http://{}'.format(idx):
            set_current(idx)


@allure.step("Try to wake up the current device！")
def wake_device(current_device):
    """

    :param current_device:
    :return:
    """
    try:
        current_device.wake()
    except AttributeError:
        pass


def app_fixture(request, device_uri, app_filepath, app_name, device_idx, init):
    """

    :param request:
    :param device_uri:
    :param app_filepath:
    :param app_name:
    :param device_idx:
    :param init:
    :return:
    """
    with allure.step("Initialize and generate APP object！"):
        logger.info("Session start test.")
        ST.THRESHOLD = 0.7
        ST.OPDELAY = 0.25
        ST.FIND_TIMEOUT = 10
        ST.FIND_TIMEOUT_TMP = 2
        ST.SNAPSHOT_QUALITY = 10

        try:
            app = None
            for uri in device_uri:
                app = my_retry_connect(uri)

                if init:
                    if str(uri).startswith('iOS'):
                        os.system('utx install {}'.format(app_filepath))
                    else:
                        install(app_filepath)
                else:
                    pass

        except Exception as e:
            port = str(device_idx).split(':')[1]
            if device_platform().lower() in 'ios':
                check_port(port=port)
            else:
                pass
            logger.error("Create app fail: {}".format(e))
            allure.attach(body='',
                          name="Create app fail: {}".format(e),
                          attachment_type=allure.attachment_type.TEXT)
            pytest.exit("Create app fail: {}".format(e))

        assert (app is not None)

        ensure_current_device(device_idx)

        logger.info("Current test platform: {}".format(device_platform()))
        logger.info("start app {0} in {1}:{2}".format(app_name, device_platform(), G.DEVICE.uuid))

        wake_device(G.DEVICE)

    def teardown_test():
        with allure.step("Teardown session"):
            stop_app(app_name)
            if init:
                if device_platform().lower() in 'ios':
                    os.system('utx uninstall {}'.format(app_name))
                else:
                    uninstall(app_name)
            else:
                pass

            if device_platform().lower() in 'ios':
                check_port(port=str(device_idx).split(':')[1])
            else:
                pass
        logger.info("Session stop test.")

    request.addfinalizer(teardown_test)

    return app


def web_info(cli_headless, cli_driver, ini_headless, ini_driver):
    """

    :param ini_driver:
    :param cli_driver:
    :param cli_headless:
    :param ini_headless:
    :return:
    """

    if cli_headless is None:
        is_headless = str_to_bool(ini_headless)
    else:
        is_headless = str_to_bool(cli_headless)

    if cli_driver is None:
        browser_driver = ini_driver
    else:
        browser_driver = cli_driver

    return is_headless, browser_driver


driver = None


def web_fixture(request, driver_path, headless):
    """

    :param request:
    :param driver_path:
    :param headless:
    :return:
    """
    with allure.step("Initialize and generate Web object！"):
        logger.info("Session start test.")
        ST.THRESHOLD = 0.7
        ST.OPDELAY = 0.25
        ST.FIND_TIMEOUT = 10
        ST.FIND_TIMEOUT_TMP = 2
        ST.SNAPSHOT_QUALITY = 10
    global driver
    if driver is None:
        if plat() == 'Windows':
            chrome_options = Options()
            chrome_options.add_argument("--remote-debugging-port=9222")
            if headless:
                chrome_options.add_argument('--headless')
            driver = webdriver.Chrome(options=chrome_options,
                                      executable_path=os.path.join(driver_path, 'chromedriver.exe'))
        elif plat() == 'Linux':
            os.system('chmod +x {}'.format(driver_path))
            chrome_options = Options()
            chrome_options.add_argument("--remote-debugging-port=9222")
            chrome_options.add_argument('--headless')
            chrome_options.add_argument('--no-sandbox')
            chrome_options.add_argument('--disable-dev-shm-usage')
            driver = webdriver.Chrome(options=chrome_options,
                                      executable_path=os.path.join(driver_path, 'chromedriver'))
        else:
            os.system('chmod +x {}'.format(driver_path))
            chrome_options = Options()
            chrome_options.add_argument("--remote-debugging-port=9222")
            if headless:
                chrome_options.add_argument('--headless')
            driver = webdriver.Chrome(options=chrome_options,
                                      executable_path=os.path.join(driver_path, 'chromedriver'))

        driver.maximize_window()
        driver.set_window_size(1920, 1080)

    driver.implicitly_wait(30)

    def fn():
        with allure.step("Teardown session"):
            driver.quit()
        logger.info("Session stop test.")

    request.addfinalizer(fn)
    return driver



#!/usr/bin/python
# encoding=utf-8

"""
@Author  :  Lijiawei
@Date    :  11/4/2021 10:14 AM
@Desc    :  Scaffold web Generator.
"""

import os
import platform
import sys

from loguru import logger


class ExtraArgument:
    create_venv = False


def init_parser_scaffold_web(subparsers):
    sub_parser_scaffold = subparsers.add_parser(
        "startproject-web", help="Create a new Web project with template structure."
    )
    sub_parser_scaffold.add_argument(
        "project_name", type=str, nargs="?", help="Specify new project name."
    )
    sub_parser_scaffold.add_argument(
        "-venv",
        dest="create_venv",
        action="store_true",
        help="Create virtual environment in the project, and install utx.",
    )
    return sub_parser_scaffold


def create_scaffold_web(project_name):
    """ Create scaffold with specified project name.
    """
    if os.path.isdir(project_name):
        logger.warning(
            f"Project folder {project_name} exists, please specify a new project name."
        )
        return 1
    elif os.path.isfile(project_name):
        logger.warning(
            f"Project name {project_name} conflicts with existed file, please specify a new one."
        )
        return 1

    logger.info(f"Create new project: {project_name}")
    print(f"Project root dir: {os.path.join(os.getcwd(), project_name)}\n")

    def create_folder(path):
        os.makedirs(path)
        msg = f"Created folder: {path}"
        print(msg)

    def create_file(path, file_content=""):
        with open(path, "w", encoding="utf-8") as f:
            f.write(file_content)
        msg = f"Created file: {path}"
        print(msg)

    create_folder(project_name)
    create_folder(os.path.join(project_name, "config"))
    create_folder(os.path.join(project_name, "logs"))
    create_folder(os.path.join(project_name, "driver"))
    create_folder(os.path.join(project_name, "report/airtest"))
    create_folder(os.path.join(project_name, "tests"))
    create_folder(os.path.join(project_name, "suites"))

    content = """venv
.idea/
.pytest_cache
*/*__pycache__/
report
debug
*.pyc
"""
    create_file(os.path.join(project_name, ".gitignore"), content)

    content = """# UTX


## 安装
- 命令行执行
```
pip install -U utx
```


## 设计理念

很大程度上借鉴了HttpRunner（优秀的框架）。不同的是，utx更着重写python，而不是写YAML文件。

- 简单是更好的
- 每个人都能用python写自动化

这就是utx的设计理念。


## 项目结构
![](https://files.mdnice.com/user/17535/555b68ce-bc10-472b-a904-b0b17203f58a.png)

- utx提供了快速创建项目的能力，也就是脚手架。
```shell
utx startproject-web project_name
```
```text
$ utx startproject demo
2021-09-01 12:39:16.491 | INFO     | utx.cli.scaffold:create_scaffold:51 - Create new project: demo
Project root dir: /PycharmProjects/demo

Created folder: demo
Created folder: demo/config
Created folder: demo/logs
Created folder: demo/driver
Created folder: demo/report/airtest
Created folder: demo/tests
Created folder: demo/suites
Created file: demo/.gitignore
Created file: demo/conftest.py
Created file: demo/pytest.ini
Created file: demo/run.py
Created file: demo/requirements.txt
Created file: demo/config/conf.py
Created file: demo/config/config.ini
Created file: demo/config/__init__.py
Created file: demo/tests/test_devices.py
Created file: demo/tests/__init__.py
Created file: demo/report/summary_template.html

```
## 调用流程图

![](https://files.mdnice.com/user/17535/b6cf34a4-c973-412e-bbef-0d56d418e895.png)

## 专注于写脚本

- 项目结构很清晰。在conftest.py进行一些初始化/参数化/清理工作，在suites/写测试脚本。
>在AirtestIDE中写好.air脚本，然后将文件放到suites文件中。
- 更注重平铺写脚本的方式，这样就离“每个人都能用python写自动化”更近一步。毕竟封装之后看着容易晕，我也晕。
- 去除掉框架的约束，给每个人写python的自由，在测试脚本里你可以尽情发挥你的代码风格，代码能力，千人千面。代价呢，就是代码质量参差不齐。

大胆写，能写，写出来，跑通，就已经是在写自动化，就已经是在创造价值了！


## 轻封装

utx尊重原生用法。

airtest的封装只通过装饰器进行了运行方式的调整，没有做任何其他的冗余修改。

- faker，造数据工具
- pytest，测试框架
- selenium，web测试框架
- airtest，自动化测试工具
- pandas、numpy，数据处理工具

安装utx，自动就把这些开源利器安装上了，无需单独安装。未来会集成更多实用工具到utx中。

utx本身是很轻的。

## 功能介绍

1. 支持android,ios,web 平台的自动化测试框架
2. 脚本批量执行
3. 每个脚本执行日志分开存放
4. 每个脚本单独生成一个html报告并在父文件夹生成一个聚合报告
5. 自定义的聚合报告，详细展示运行结果
6. 重试机制，运行失败自动重跑，可自定义重跑次数
7. 自定义脚本运行，可选择部分模块单独运行
8. 自带脚手架工具可以快速生成框架目录


## 使用说明
- 下载好chrome浏览器对应的驱动，放到driver文件夹中
> 注意：文件名规范 【Linux/Mac：chromedriver】【Windows：chromedriver.exe】
- 只需在配置文件中填好相关内容，即可运行！
```ini
[web_info]
;被测的主页url
url = https://www.baidu.com/
;是否无界面运行
headless = False

[reruns]
;失败后再次运行次数，默认1次
times = 1

[mode]
;0表示 运行[suites][cases]选择的用例  1表示运行全部用例
is_all = 0

[suites]
;填写用例的关键字
cases = chrome

```


- 启动命令
1. 普通启动
```shell
python run.py
```
2. 参数化启动
```shell
python run.py --headless=True --driver=/Users/admin/driverpath
```
>参数优先级大于配置文件

## 运行结果
- 报告展示

![](https://files.mdnice.com/user/17535/29b7a536-9e30-45e3-b7ed-727a2091b910.png)

"""

    create_file(os.path.join(project_name, "README.md"), content)

    content = """#!/usr/bin/python
# encoding=utf-8

\"\"\" Can only be modified by the administrator. Only fixtures are provided.
\"\"\"

import pytest

from airtest.core.settings import Settings as ST
from utx.core.utils.hook import web_info, web_fixture
from config.conf import *
from run import cli_headless, cli_driver_path

ST.PROJECT_ROOT = BASE_DIR
is_headless, browser_driver_path = web_info(cli_headless=cli_headless,
                                            cli_driver=cli_driver_path,
                                            ini_headless=ini.getvalue('web_info', 'headless'),
                                            ini_driver=DRIVER_PATH)


@pytest.fixture(scope="session", autouse=True)
def web_init(request):
    driver = web_fixture(request, driver_path=browser_driver_path, headless=is_headless)
    return driver
"""
    create_file(os.path.join(project_name, "conftest.py"), content)

    content = """[pytest]
log_cli=true
log_cli_level=INFO
log_format = %(asctime)s [%(levelname)-5s] %(name)s %(funcName)s : %(message)s
log_date_format = %Y-%m-%d %H:%M:%S

# show all extra test summary info
addopts = -ra
testpaths = tests
python_files = test_browsers.py
"""
    create_file(os.path.join(project_name, "pytest.ini"), content)

    content = """#!/usr/bin/python
# encoding=utf-8
\"\"\" Can only be modified by the administrator. Only Runner are provided.
\"\"\"

import os
import pytest
from utx.cli.cli import cli_web_env

from config.conf import ALLURE_REPORT_PATH
from utx.core.utils.tools import allure_report

cli_headless, cli_driver_path = cli_web_env()

if __name__ == '__main__':
    report_path = ALLURE_REPORT_PATH + os.sep + "result"
    report_html_path = ALLURE_REPORT_PATH + os.sep + "html"
    pytest.main(["-s", "--alluredir", report_path, "--clean-alluredir"])
    allure_report(report_path, report_html_path)
"""

    create_file(os.path.join(project_name, "run.py"), content)

    content = """# Customize third-parties
# pip install --default-timeout=6000 -i https://pypi.tuna.tsinghua.edu.cn/simple -r requirements.txt

"""
    create_file(os.path.join(project_name, "requirements.txt"), content)

    content = """#!/usr/bin/python
# encoding=utf-8
\"\"\" Can only be modified by the administrator. Only Path are provided.
\"\"\"
import os

# 项目目录
from utx.core.utils.tools import ReadConfig

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

# 配置文件
INI_PATH = os.path.join(BASE_DIR, 'config', 'config.ini')

# 测试用例目录
CASE_PATH = os.path.join(BASE_DIR, 'suites')
if not os.path.exists(CASE_PATH):
    os.mkdir(CASE_PATH)
# 日志目录
LOG_PATH = os.path.join(BASE_DIR, 'logs')
if not os.path.exists(LOG_PATH):
    os.mkdir(LOG_PATH)
# 驱动目录
DRIVER_PATH = os.path.join(BASE_DIR, 'driver')
if not os.path.exists(DRIVER_PATH):
    os.mkdir(DRIVER_PATH)
# allure报告目录
ALLURE_REPORT_PATH = os.path.join(BASE_DIR, 'report')
if not os.path.exists(ALLURE_REPORT_PATH):
    os.mkdir(ALLURE_REPORT_PATH)

# airtest报告目录
AIRTEST_REPORT_PATH = os.path.join(BASE_DIR, 'report', 'airtest')
if not os.path.exists(AIRTEST_REPORT_PATH):
    os.mkdir(AIRTEST_REPORT_PATH)

ini = ReadConfig(INI_PATH)

if __name__ == '__main__':
    pass
"""

    create_file(os.path.join(project_name, "config", "conf.py"), content)

    content = """[web_info]
;被测的主页url
url = https://www.baidu.com/
;是否无界面运行
headless = False

[reruns]
;失败后再次运行次数，默认1次
times = 1

[mode]
;0表示 运行[suites][cases]选择的用例  1表示运行全部用例
is_all = 0

[suites]
;填写用例的关键字
cases = chrome
"""
    create_file(os.path.join(project_name, "config", "config.ini"), content)
    create_file(os.path.join(project_name, "config", "__init__.py"))

    content = """#!/usr/bin/python
# encoding=utf-8
\"\"\" Can only be modified by the administrator. Only Cases are provided.
\"\"\"

import allure
import pytest
from utx.core.css.custom_report import gen_report
from utx.core.utils.tools import decryption, selector

from config.conf import *
from utx.core.utils.runner import run_web_air

cases_list = sorted(os.listdir(CASE_PATH))
results = []
result_list = []

status = ini.getvalue('mode', 'is_all')
flag = ini.getvalue('suites', 'cases')
home_url = ini.getvalue('web_info', 'url')
times = int(ini.getvalue('reruns', 'times'))
cases = selector(status=status, flag=flag, cases_list=cases_list)


@allure.feature("Web automation test")
class TestWeb:
    @pytest.mark.flaky(reruns=times)
    @pytest.mark.parametrize("case", cases)
    @allure.story("Web automation test")
    @allure.title("Current browser:chrome,Running case:{case}")
    @allure.description("APP automation test")
    @allure.severity(allure.severity_level.TRIVIAL)
    @allure.tag("Smoke testing")
    def test_ui_web(self, case, web_init):
        driver = web_init
        driver.get(home_url)
        result, dev = run_web_air(case=case, log_path=LOG_PATH, case_path=CASE_PATH,
                                  base_dir=BASE_DIR,
                                  static_dir=decryption('aHR0cHM6Ly9jZG4uanNkZWxpdnIubmV0L2doL29wZW51dHgvc3RhdGljLw=='))
        result_list.append(result)
        summary_info = {'devices': dev, 'result': result_list}
        results.append(summary_info)
        assert result['result'] is True

    @allure.story("Web automation test")
    @allure.title("Generate test report of running device")
    @allure.description("Web automation test")
    @allure.severity(allure.severity_level.TRIVIAL)
    @allure.tag("Generate report")
    def test_report(self):
        print(results)
        gen_report(results=results, report_path=AIRTEST_REPORT_PATH)


if __name__ == "__main__":
    pytest.main(["-s", "test_browsers.py"])
"""

    create_file(os.path.join(project_name, "tests", "test_browsers.py"), content)
    create_file(os.path.join(project_name, "tests", "__init__.py"))

    if ExtraArgument.create_venv:
        os.chdir(project_name)
        print("\nCreating virtual environment")
        os.system("python -m venv venv")
        print("Created virtual environment: venv")

        print("Installing utx")
        if platform.system().lower() == 'windows':
            os.chdir("venv")
            os.chdir("Scripts")
            os.system("pip install utx")
        elif platform.system().lower() == 'linux':
            os.chdir("venv")
            os.chdir("bin")
            os.system("pip install utx")
        elif platform.system().lower() == 'mac':
            os.chdir("venv")
            os.chdir("bin")
            os.system("pip install utx")


def main_scaffold_web(args):
    ExtraArgument.create_venv = args.create_venv
    sys.exit(create_scaffold_web(args.project_name))



#!/usr/bin/python
# encoding=utf-8

"""
@Author  :  Lijiawei
@Date    :  2021/12/11 11:15 上午 
@Desc    :  pg_2 line.
"""
from utx.mod.platform import TextProcessor


@TextProcessor.plugin_register('pg2')
class CleanMarkdownItalic(object):
    @staticmethod
    def process(text):
        return text.replace('--', '')



#!/usr/bin/python
# encoding=utf-8

"""
@Author  :  Lijiawei
@Date    :  8/31/2021 6:42 PM
@Desc    :  Runner line.
"""
import time
import unittest
import sys
import six
import re
import shutil
import traceback
import warnings
from io import open
from airtest.core.api import G, auto_setup, log, stop_app
from airtest.core.settings import Settings as ST
from airtest.utils.compat import script_dir_name, script_log_dir
from copy import copy

from argparse import *

from airtest.core.android.adb import ADB
from airtest.core.api import device, start_app
from airtest.core.error import AirtestError
from utx.selenium.exceptions import AirtestSeleniumException
from poco.exceptions import PocoException

from utx.core.css import report
import os


class AirtestCase(unittest.TestCase):
    PROJECT_ROOT = "."
    SCRIPTEXT = ".air"
    TPLEXT = ".png"

    @classmethod
    def setUpClass(cls):
        cls.args = args

        setup_by_args(args)

        # setup script exec scope
        cls.scope = copy(globals())
        cls.scope["exec_script"] = cls.exec_other_script

    def setUp(self):
        if self.args.log and self.args.recording:
            for dev in G.DEVICE_LIST:
                try:
                    dev.start_recording()
                except:
                    traceback.print_exc()

    def tearDown(self):
        if self.args.log and self.args.recording:
            for k, dev in enumerate(G.DEVICE_LIST):
                # 录屏文件保存的命名规则：
                # 如果未指定文件名，只传了--recording，就默认用recording_手机序列号.mp4来命名
                # 如果指定了文件名--recording test.mp4，且超过一台手机，就命名为 手机序列号_test.mp4
                # 否则直接用指定的文件名保存录屏，必须是mp4结尾
                try:
                    if isinstance(self.args.recording, six.string_types) and self.args.recording.endswith(".mp4"):
                        basename = os.path.basename(self.args.recording)
                        output_name = dev.serialno + "_" + basename if len(G.DEVICE_LIST) > 1 else basename
                    else:
                        output_name = "recording_{serialno}.mp4".format(serialno=dev.serialno)
                    output = os.path.join(self.args.log, output_name)
                    dev.stop_recording(output)
                except:
                    traceback.print_exc()

    def runTest(self):
        scriptpath, pyfilename = script_dir_name(self.args.script)
        pyfilepath = os.path.join(scriptpath, pyfilename)
        pyfilepath = os.path.abspath(pyfilepath)
        self.scope["__file__"] = pyfilepath
        with open(pyfilepath, 'r', encoding="utf8") as f:
            code = f.read()
        pyfilepath = pyfilepath.encode(sys.getfilesystemencoding())

        try:
            exec(compile(code.encode("utf-8"), pyfilepath, 'exec'), self.scope)
        except Exception as err:
            log(err, desc="Final Error", snapshot=True)
            six.reraise(*sys.exc_info())

    @classmethod
    def exec_other_script(cls, scriptpath):
        """run other script in test script"""

        warnings.simplefilter("always")
        warnings.warn("please use using() api instead.", PendingDeprecationWarning)

        def _sub_dir_name(scriptname):
            dirname = os.path.splitdrive(os.path.normpath(scriptname))[-1]
            dirname = dirname.strip(os.path.sep).replace(os.path.sep, "_").replace(cls.SCRIPTEXT, "_sub")
            return dirname

        def _copy_script(src, dst):
            if os.path.isdir(dst):
                shutil.rmtree(dst, ignore_errors=True)
            os.mkdir(dst)
            for f in os.listdir(src):
                srcfile = os.path.join(src, f)
                if not (os.path.isfile(srcfile) and f.endswith(cls.TPLEXT)):
                    continue
                dstfile = os.path.join(dst, f)
                shutil.copy(srcfile, dstfile)

        # find script in PROJECT_ROOT
        scriptpath = os.path.join(ST.PROJECT_ROOT, scriptpath)
        # copy submodule's images into sub_dir
        sub_dir = _sub_dir_name(scriptpath)
        sub_dirpath = os.path.join(cls.args.script, sub_dir)
        _copy_script(scriptpath, sub_dirpath)
        # read code
        pyfilename = os.path.basename(scriptpath).replace(cls.SCRIPTEXT, ".py")
        pyfilepath = os.path.join(scriptpath, pyfilename)
        pyfilepath = os.path.abspath(pyfilepath)
        with open(pyfilepath, 'r', encoding='utf8') as f:
            code = f.read()
        # replace tpl filepath with filepath in sub_dir
        code = re.sub(r"[\'\"](\w+.png)[\'\"]", r"\"%s/\g<1>\"" % sub_dir, code)
        exec(compile(code.encode("utf8"), pyfilepath, 'exec'), cls.scope)


def setup_by_args(args):
    # init devices
    if isinstance(args.device, list):
        devices = args.device
    elif args.device:
        devices = [args.device]
    else:
        devices = []
        print("do not connect device")

    # set base dir to find tpl
    dirpath, _ = script_dir_name(args.script)

    # set log dir
    if args.log:
        args.log = script_log_dir(dirpath, args.log)
        print("save log in '%s'" % args.log)
    else:
        print("do not save log")

    # set snapshot quality
    if args.compress:
        compress = args.compress
    else:
        compress = ST.SNAPSHOT_QUALITY

    if args.no_image:
        ST.SAVE_IMAGE = False

    # guess project_root to be basedir of current .air path
    project_root = os.path.dirname(args.script) if not ST.PROJECT_ROOT else None

    auto_setup(dirpath, devices, args.log, project_root, compress)


def run_script(parsed_args, testcase_cls=AirtestCase):
    global args  # make it global deliberately to be used in AirtestCase & test scripts
    args = parsed_args
    suite = unittest.TestSuite()
    suite.addTest(testcase_cls())
    result = unittest.TextTestRunner(verbosity=0).run(suite)
    # if not result.wasSuccessful():
    #     sys.exit(-1)


def custom_resize_method(w, h, sch_resolution, src_resolution):
    """
    分辨率适配规则
    :param w:
    :param h:
    :param sch_resolution:
    :param src_resolution:
    :return:设备分辨率
    """
    return int(w), int(h)


def run_air(devices, case, app_name, log_path, case_path, base_dir, static_dir='https://cdn.jsdelivr.net/gh/openutx/static/'):
    """

    :param devices: 读取的设备id
    :param case: case list
    :param log_path: log_path
    :param case_path: case_path
    :param base_dir: base_dir
    :param app_name: app_name
    :param static_dir: 为空时使用本地样式文件

    :return:
    """
    try:
        dev = device().device_info['name']
    except (TypeError, AttributeError):
        serialno = device().serialno
        dev = ADB(serialno=serialno).get_model()

    stop_app(app_name)
    time.sleep(2)
    start_app(app_name)
    time.sleep(2)

    if os.path.isdir(log_path):
        pass
    else:
        os.makedirs(log_path)
        print(str(log_path) + 'is created')
    f = case
    if f.endswith(".air"):

        airName = f
        script = os.path.join(case_path, f)

        print(script)
        log = os.path.join(log_path + '/' + dev + '/' + airName.replace('.air', ''))

        run_log = script + '/' + 'log' + '/' + dev
        if not os.path.exists(run_log):
            os.makedirs(run_log)
        print(log)
        if os.path.isdir(log):
            pass
        else:
            os.makedirs(log)
            print(str(log) + 'is created')
        output_file = log + '/' + 'log.html'
        args = Namespace(device=devices, log=run_log, recording=None, script=script, compress=20, no_image=False)
        try:
            run_script(args, AirtestCase)
        except (AirtestError, PocoException):
            pass
        finally:
            rpt = report.LogToHtml(script, run_log, script_name=f.replace(".air", ".py"), static_root=static_dir)
            rpt.report("log_template.html", output_file=output_file, base_dir=base_dir)
            result = {"name": airName.replace('.air', ''), "result": rpt.test_result}

        return result, dev


def run_web_air(case, log_path, case_path, base_dir, static_dir='https://cdn.jsdelivr.net/gh/openutx/static/'):
    """

    :param case:
    :param log_path:
    :param case_path:
    :param base_dir:
    :param static_dir:
    :return:
    """
    dev = 'chrome'
    time.sleep(2)

    if os.path.isdir(log_path):
        pass
    else:
        os.makedirs(log_path)
        print(str(log_path) + 'is created')
    f = case
    if f.endswith(".air"):

        airName = f
        script = os.path.join(case_path, f)

        print(script)
        log = os.path.join(log_path + '/' + dev + '/' + airName.replace('.air', ''))

        run_log = script + '/' + 'log' + '/' + dev
        if not os.path.exists(run_log):
            os.makedirs(run_log)
        print(log)
        if os.path.isdir(log):
            pass
        else:
            os.makedirs(log)
            print(str(log) + 'is created')
        output_file = log + '/' + 'log.html'
        args = Namespace(device=None, log=run_log, recording=None, script=script, compress=20, no_image=False)
        try:
            run_script(args, AirtestCase)
        except (AirtestError, PocoException, AirtestSeleniumException):
            pass
        finally:
            rpt = report.LogToHtml(script, run_log, script_name=f.replace(".air", ".py"), static_root=static_dir,
                                   plugins=['utx.selenium.report', 'poco.utils.airtest.report'])
            rpt.report("log_template.html", output_file=output_file, base_dir=base_dir)
            result = {"name": airName.replace('.air', ''), "result": rpt.test_result}

        return result, dev



#!/usr/bin/python
# encoding=utf-8

"""
@Author  :  Lijiawei
@Date    :  11/02/2021 8:16 PM
@Desc    :  exceptions line.
"""

from __future__ import unicode_literals

import utx.selenium.utils.six as six


def to_text(val):
    if not isinstance(val, six.text_type):
        return val.decode('utf-8')
    return val


class AirtestSeleniumException(Exception):
    """
    Base class for errors and exceptions of Airtest-Selenium
    """

    def __init__(self, message=None):
        super(AirtestSeleniumException, self).__init__(message)
        self.message = message

    def __str__(self):
        if six.PY2:
            if isinstance(self.message, six.text_type):
                return self.message.encode("utf-8")
            else:
                return self.message
        else:
            if isinstance(self.message, bytes):
                return self.message.decode('utf-8')
            else:
                return self.message

    __repr__ = __str__


class IsNotTemplateError(AirtestSeleniumException):
    """
    Base class for errors and exceptions of Airtest-Selenium
    """

    def __init__(self, message=None):
        super(IsNotTemplateError, self).__init__(message)

    def __str__(self):
        if six.PY2:
            if isinstance(self.message, six.text_type):
                return self.message.encode("utf-8")
            else:
                return self.message
        else:
            if isinstance(self.message, bytes):
                return self.message.decode('utf-8')
            else:
                return self.message

    __repr__ = __str__



#!/usr/bin/python
# encoding=utf-8

"""
@Author  :  Lijiawei
@Date    :  11/02/2021 3:36 PM
@Desc    :  proxy line
"""

from selenium.webdriver import Chrome, ActionChains, Firefox, Remote
from selenium.webdriver.remote.webelement import WebElement
from selenium.common.exceptions import NoSuchElementException
from selenium.webdriver.chrome.options import Options
from airtest.core.settings import Settings as ST
from airtest.core.helper import logwrap
from airtest import aircv
from airtest.core.cv import Template
from utx.selenium.utils.airtest_api import loop_find
from utx.selenium.exceptions import IsNotTemplateError
from airtest.aircv import get_resolution
from pynput.mouse import Controller, Button
from airtest.core.error import TargetNotFoundError

import os
import time
import sys

from utx.core.utils.tools import plat


class WebChrome(Chrome):
    chrome_options = Options()
    chrome_options.add_experimental_option("debuggerAddress", "127.0.0.1:9222")
    if plat() == 'Windows':
        driver_name = 'chromedriver.exe'
    else:
        driver_name = 'chromedriver'
    driver_path = os.path.join(ST.PROJECT_ROOT, 'driver', '{}'.format(driver_name))

    def __init__(self, executable_path=driver_path, port=0,
                 options=chrome_options, service_args=None,
                 desired_capabilities=None, service_log_path=None,
                 chrome_options=None):

        # todo 适配selenium 4.0.0
        # def __init__(self, executable_path="chromedriver", port=0,
        #              options: Options = chrome_options, service_args=None,
        #              desired_capabilities=None, service_log_path=None,
        #              chrome_options=None, service=Service(executable_path=driver_path), keep_alive=None):

        if "darwin" in sys.platform:
            os.environ['PATH'] += ":/Applications/AirtestIDE.app/Contents/Resources/selenium_plugin"
        super(WebChrome, self).__init__(chrome_options=chrome_options, executable_path=executable_path,
                                        port=port, options=options, service_args=service_args,
                                        service_log_path=service_log_path, desired_capabilities=desired_capabilities)
        self.father_number = {0: 0}
        self.action_chains = ActionChains(self)
        self.number = 0
        self.mouse = Controller()
        self.operation_to_func = {"elementsD": self.find_any_element, "xpath": self.find_element_by_xpath,
                                  "id": self.find_element_by_id,
                                  "name": self.find_element_by_name, "css": self.find_element_by_css_selector}

    def loop_find_element(self, func, text, timeout=10, interval=0.5):
        """
        Loop to find the target web element by func.

        Args:
            func: function to find element
            text: param of function
            timeout: time to find the element
            interval: interval between operation
        Returns:
            element that been found
        """
        start_time = time.time()
        while True:
            try:
                element = func(text)
            except NoSuchElementException:
                print("Element not found!")
                # 超时则raise，未超时则进行下次循环:
                if (time.time() - start_time) > timeout:
                    # try_log_screen(screen)
                    raise NoSuchElementException(
                        'Element %s not found in screen' % text)
                else:
                    time.sleep(interval)
            else:
                return element

    def loop_find_element_noExc(self, func, text, timeout=3, interval=0.5):
        """
        Loop to find the target web element by func.

        Args:
            func: function to find element
            text: param of function
            timeout: time to find the element
            interval: interval between operation
        Returns:
            element that been found
        """
        start_time = time.time()
        while True:
            try:
                element = func(text)
            except NoSuchElementException:
                print("Element not found!")
                # 超时则raise，未超时则进行下次循环:
                if (time.time() - start_time) > timeout:
                    # try_log_screen(screen)
                    return None
                else:
                    time.sleep(interval)
            else:
                print('element found')
                return element

    @logwrap
    def find_any_element(self, elementsD):
        """
        Find the web element by the indicated parameters.

        Args:
            elementsD: a dictionary with keys = by's, values=value
        Returns:
            Web element of current page.
        """
        web_element = None
        for key in elementsD:
            value = elementsD[key]
            print(value)
            if key.upper() == 'ID':
                web_element = self.loop_find_element_noExc(super().find_element_by_id, value)
            elif key.upper() == 'XPATH':
                web_element = self.loop_find_element_noExc(super().find_element_by_xpath, value)
            elif key.upper() == 'CSS':
                web_element = self.loop_find_element_noExc(super().find_element_by_css_selector, value)
            elif key.upper() == 'NAME':
                web_element = self.loop_find_element_noExc(super().find_element_by_name, value)
            elif key.upper() == 'LINKTEXT':
                web_element = self.loop_find_element_noExc(super().find_element_by_link_text, value)
            elif key.upper() == 'CLASSNAME':
                web_element = self.loop_find_element_noExc(super().find_element_by_class_name, value)
            elif key.upper() == 'PARTIALLINKTEXT':
                web_element = self.loop_find_element_noExc(super().find_element_by_partial_link_text, value)
            elif key.upper() == 'TAGNAME':
                web_element = self.loop_find_element_noExc(super().find_element_by_tag_name, value)
                # check by position/ picture / visual testing
            if web_element is not None:
                break
        if web_element is not None:
            log_res = self._gen_screen_log(web_element)
            return Element(web_element, log_res)
        raise NoSuchElementException('Element not found in screen')

    @logwrap
    def find_element_by_xpath(self, xpath):
        """
        Find the web element by xpath.

        Args:
            xpath: find the element by xpath.
        Returns:
            Web element of current page.
        """
        web_element = self.loop_find_element(super(WebChrome, self).find_element_by_xpath, xpath)
        # web_element = super(WebChrome, self).find_element_by_xpath(xpath)
        log_res = self._gen_screen_log(web_element)
        return Element(web_element, log_res)

    @logwrap
    def find_element_by_id(self, id):
        """
        Find the web element by id.

        Args:
            id: find the element by attribute id.
        Returns:
            Web element of current page.
        """
        web_element = self.loop_find_element(super(WebChrome, self).find_element_by_id, id)
        log_res = self._gen_screen_log(web_element)
        return Element(web_element, log_res)

    @logwrap
    def find_element_by_css_selector(self, css_selector):
        """
        Find the web element by css_selector.

        Args:
            css_selector: find the element by attribute css_selector.
        Returns:
            Web element of current page.
        """
        web_element = self.loop_find_element(super(WebChrome, self).find_element_by_css_selector, css_selector)
        log_res = self._gen_screen_log(web_element)
        return Element(web_element, log_res)

    @logwrap
    def find_element_by_name(self, name):
        """
        Find the web element by name.

        Args:
            name: find the element by attribute name.
        Returns:
            Web element of current page.
        """
        web_element = self.loop_find_element(super(WebChrome, self).find_element_by_name, name)
        log_res = self._gen_screen_log(web_element)
        return Element(web_element, log_res)

    @logwrap
    def switch_to_new_tab(self):
        """
        Switch to the new tab.
        """
        _father = self.number
        self.number = len(self.window_handles) - 1
        self.father_number[self.number] = _father
        self.switch_to.window(self.window_handles[self.number])
        self._gen_screen_log()
        time.sleep(0.5)

    @logwrap
    def switch_to_previous_tab(self):
        """
        Switch to the previous tab(which to open current tab).
        """
        self.number = self.father_number[self.number]
        self.switch_to.window(self.window_handles[self.number])
        self._gen_screen_log()
        time.sleep(0.5)

    @logwrap
    def airtest_touch(self, v):
        """
        Perform the touch action on the current page by image identification.

        Args:
            v: target to touch, either a Template instance or absolute coordinates (x, y)
        Returns:
            Finial position to be clicked.
        """
        # 调用airtest opencv 图像识别能力
        if isinstance(v, Template):
            _pos = loop_find(v, timeout=ST.FIND_TIMEOUT, driver=self)
        else:
            _pos = v
        x, y = _pos
        # self.action_chains.move_to_element_with_offset(root_element, x, y)
        # self.action_chains.click()
        pos = self._get_left_up_offset()
        pos = (pos[0] + x, pos[1] + y)
        self._move_to_pos(pos)
        self._click_current_pos()
        time.sleep(1)
        return _pos

    @logwrap
    def assert_template(self, v, msg=""):
        """
        Assert target exists on the current page.

        Args:
            v: target to touch, either a Template instance
        Raise:
            AssertionError - if target not found.
        Returns:
            Position of the template.
        """
        if isinstance(v, Template):
            try:
                pos = loop_find(v, timeout=ST.FIND_TIMEOUT, driver=self)
            except TargetNotFoundError:
                raise AssertionError("Target template not found on screen.")
            else:
                return pos
        else:
            raise IsNotTemplateError("args is not a template")

    @logwrap
    def assert_exist(self, param, operation, msg=""):
        """
        Assert element exist.

        Args:
            operation: the method that to find the element.
            param: the param of method.
        Raise:
            AssertionError - if assertion failed.
        """
        try:
            func = self.operation_to_func[operation]
        except Exception:
            raise AssertionError("There was no operation: %s" % operation)
        try:
            func(param)
        except Exception as e:
            raise AssertionError("Target element not find.")

    @logwrap
    def get(self, address):
        """
        Access the web address.

        Args:
            address: the address that to accesss
        """
        super(WebChrome, self).get(address)
        time.sleep(2)

    @logwrap
    def back(self):
        """
        Back to last page.
        """
        super(WebChrome, self).back()
        self._gen_screen_log()
        time.sleep(1)

    @logwrap
    def forward(self):
        """
        Forward to next page.
        """
        super(WebChrome, self).forward()
        self._gen_screen_log()
        time.sleep(1)

    @logwrap
    def snapshot(self, filename=None):
        self._gen_screen_log(filename=filename)

    @logwrap
    def _gen_screen_log(self, element=None, filename=None, ):
        if ST.LOG_DIR is None:
            return None
        if filename:
            self.screenshot(filename)

        # 忽略警告 .jpg
        jpg_file_name = str(int(time.time())) + '.png'
        # 适配utx报告路径
        jpg_path = os.path.join(ST.LOG_DIR, jpg_file_name)
        print("this is jpg path:", jpg_path)
        self.screenshot(jpg_path)
        saved = {"screen": jpg_path}
        if element:
            size = element.size
            location = element.location
            x = size['width'] / 2 + location['x']
            y = size['height'] / 2 + location['y']
            if "darwin" in sys.platform:
                x, y = x * 2, y * 2
            saved.update({"pos": [[x, y]]})
        return saved

    def screenshot(self, file_path=None):
        if file_path:
            self.save_screenshot(file_path)
        else:
            if not ST.LOG_DIR:
                file_path = "temp.png"
            else:
                file_path = os.path.join(ST.LOG_DIR, "temp.png")
            self.save_screenshot(file_path)
            screen = aircv.imread(file_path)
            return screen

    def _get_left_up_offset(self):
        window_pos = self.get_window_position()
        window_size = self.get_window_size()
        mouse = Controller()
        screen = self.screenshot()
        screen_size = get_resolution(screen)
        offset = window_size["width"] - \
                 screen_size[0], window_size["height"] - screen_size[1]
        pos = (int(offset[0] / 2 + window_pos['x']),
               int(offset[1] + window_pos['y'] - offset[0] / 2))
        return pos

    def _move_to_pos(self, pos):
        self.mouse.position = pos

    def _click_current_pos(self):
        self.mouse.click(Button.left, 1)

    def to_json(self):
        # add this method for json encoder in logwrap
        return repr(self)


class WebRemote(Remote):

    def __init__(self, command_executor='http://127.0.0.1:4444/wd/hub',
                 desired_capabilities=None, browser_profile=None, proxy=None,
                 keep_alive=False, file_detector=None, options=None):
        super(WebRemote, self).__init__(command_executor=command_executor,
                                        desired_capabilities=desired_capabilities, browser_profile=browser_profile,
                                        proxy=proxy,
                                        keep_alive=keep_alive, file_detector=file_detector, options=options)

        self.father_number = {0: 0}
        self.action_chains = ActionChains(self)
        self.number = 0
        self.mouse = Controller()
        self.operation_to_func = {"xpath": self.find_element_by_xpath, "id": self.find_element_by_id,
                                  "name": self.find_element_by_name, "css": self.find_element_by_css_selector}

    def loop_find_element(self, func, text, timeout=10, interval=0.5):
        """
        Loop to find the target web element by func.

        Args:
            func: function to find element
            text: param of function
            timeout: time to find the element
            interval: interval between operation
        Returns:
            element that been found
        """
        start_time = time.time()
        while True:
            try:
                element = func(text)
            except NoSuchElementException:
                print("Element not found!")
                # 超时则raise，未超时则进行下次循环:
                if (time.time() - start_time) > timeout:
                    # try_log_screen(screen)
                    raise NoSuchElementException('Element %s not found in screen' % text)
                else:
                    time.sleep(interval)
            else:
                return element

    @logwrap
    def find_element_by_xpath(self, xpath):
        """
        Find the web element by xpath.

        Args:
            xpath: find the element by xpath.
        Returns:
            Web element of current page.
        """
        web_element = self.loop_find_element(super(WebRemote, self).find_element_by_xpath, xpath)
        log_res = self._gen_screen_log(web_element)
        return Element(web_element, log_res)

    @logwrap
    def find_element_by_id(self, id):
        """
        Find the web element by id.

        Args:
            id: find the element by attribute id.
        Returns:
            Web element of current page.
        """
        web_element = self.loop_find_element(super(WebRemote, self).find_element_by_id, id)
        log_res = self._gen_screen_log(web_element)
        return Element(web_element, log_res)

    @logwrap
    def find_element_by_css_selector(self, css_selector):
        """
        Find the web element by css_selector.

        Args:
            css_selector: find the element by attribute css_selector.
        Returns:
            Web element of current page.
        """
        web_element = self.loop_find_element(super(WebRemote, self).find_element_by_css_selector, css_selector)
        log_res = self._gen_screen_log(web_element)
        return Element(web_element, log_res)

    @logwrap
    def find_element_by_name(self, name):
        """
        Find the web element by name.

        Args:
            name: find the element by attribute name.
        Returns:
            Web element of current page.
        """
        web_element = self.loop_find_element(super(WebRemote, self).find_element_by_name, name)
        log_res = self._gen_screen_log(web_element)
        return Element(web_element, log_res)

    @logwrap
    def switch_to_new_tab(self):
        """
        Switch to the new tab.
        """
        _father = self.number
        self.number = len(self.window_handles) - 1
        self.father_number[self.number] = _father
        self.switch_to.window(self.window_handles[self.number])
        self._gen_screen_log()
        time.sleep(0.5)

    @logwrap
    def switch_to_previous_tab(self):
        """
        Switch to the previous tab(which to open current tab).
        """
        self.number = self.father_number[self.number]
        self.switch_to.window(self.window_handles[self.number])
        self._gen_screen_log()
        time.sleep(0.5)

    @logwrap
    def airtest_touch(self, v):
        """
        Perform the touch action on the current page by image identification.

        Args:
            v: target to touch, either a Template instance or absolute coordinates (x, y)
        Returns:
            Finial position to be clicked.
        """
        if isinstance(v, Template):
            _pos = loop_find(v, timeout=ST.FIND_TIMEOUT, driver=self)
        else:
            _pos = v
        x, y = _pos
        # self.action_chains.move_to_element_with_offset(root_element, x, y)
        # self.action_chains.click()
        pos = self._get_left_up_offset()
        pos = (pos[0] + x, pos[1] + y)
        self._move_to_pos(pos)
        self._click_current_pos()
        time.sleep(1)
        return _pos

    @logwrap
    def assert_template(self, v, msg=""):
        """
        Assert target exists on the current page.

        Args:
            v: target to touch, either a Template instance
        Raise:
            AssertionError - if target not found.
        Returns:
            Position of the template.
        """
        if isinstance(v, Template):
            try:
                pos = loop_find(v, timeout=ST.FIND_TIMEOUT, driver=self)
            except TargetNotFoundError:
                raise AssertionError("Target template not found on screen.")
            else:
                return pos
        else:
            raise IsNotTemplateError("args is not a template")

    @logwrap
    def assert_exist(self, param, operation, msg=""):
        """
        Assert element exist.

        Args:
            operation: the method that to find the element.
            param: the param of method.
        Raise:
            AssertionError - if assertion failed.
        """
        try:
            func = self.operation_to_func[operation]
        except Exception:
            raise AssertionError("There was no operation: %s" % operation)
        try:
            func(param)
        except Exception as e:
            raise AssertionError("Target element not find.")

    @logwrap
    def get(self, address):
        """
        Access the web address.

        Args:
            address: the address that to accesss
        """
        super(WebRemote, self).get(address)
        time.sleep(2)

    @logwrap
    def back(self):
        """
        Back to last page.
        """
        super(WebRemote, self).back()
        self._gen_screen_log()
        time.sleep(1)

    @logwrap
    def forward(self):
        """
        Forward to next page.
        """
        super(WebRemote, self).forward()
        self._gen_screen_log()
        time.sleep(1)

    @logwrap
    def snapshot(self, filename=None):
        self._gen_screen_log(filename=filename)

    @logwrap
    def _gen_screen_log(self, element=None, filename=None, ):
        if ST.LOG_DIR is None:
            return None
        if filename:
            self.screenshot(filename)
        jpg_file_name = str(int(time.time())) + '.png'
        jpg_path = os.path.join(ST.LOG_DIR, jpg_file_name)
        print("this is jpg path:", jpg_path)
        self.screenshot(jpg_path)
        saved = {"screen": jpg_path}
        if element:
            size = element.size
            location = element.location
            x = size['width'] / 2 + location['x']
            y = size['height'] / 2 + location['y']
            if "darwin" in sys.platform:
                x, y = x * 2, y * 2
            saved.update({"pos": [[x, y]]})
        return saved

    def screenshot(self, file_path=None):
        if file_path:
            self.save_screenshot(file_path)
        else:
            if not ST.LOG_DIR:
                file_path = "temp.png"
            else:
                file_path = os.path.join(ST.LOG_DIR, "temp.png")
            self.save_screenshot(file_path)
            screen = aircv.imread(file_path)
            return screen

    def _get_left_up_offset(self):
        window_pos = self.get_window_position()
        window_size = self.get_window_size()
        mouse = Controller()
        screen = self.screenshot()
        screen_size = get_resolution(screen)
        offset = window_size["width"] - \
                 screen_size[0], window_size["height"] - screen_size[1]
        pos = (int(offset[0] / 2 + window_pos['x']),
               int(offset[1] + window_pos['y'] - offset[0] / 2))
        return pos

    def _move_to_pos(self, pos):
        self.mouse.position = pos

    def _click_current_pos(self):
        self.mouse.click(Button.left, 1)

    def to_json(self):
        # add this method for json encoder in logwrap
        return repr(self)


class WebFirefox(Firefox):

    def __init__(self, firefox_profile=None, firefox_binary=None,
                 timeout=30, capabilities=None, proxy=None,
                 executable_path="geckodriver", options=None, firefox_options=None,
                 service_args=None, desired_capabilities=None, log_path=None):
        print("Please make sure your geckodriver is in your path before proceeding using this driver")
        super(WebFirefox, self).__init__(firefox_profile=firefox_profile, firefox_binary=firefox_binary,
                                         timeout=timeout, capabilities=capabilities, proxy=proxy,
                                         executable_path=executable_path, options=options,
                                         firefox_options=firefox_options,
                                         service_args=service_args, desired_capabilities=desired_capabilities,
                                         log_path=log_path)
        self.father_number = {0: 0}
        self.action_chains = ActionChains(self)
        self.number = 0
        self.mouse = Controller()
        self.operation_to_func = {"xpath": self.find_element_by_xpath, "id": self.find_element_by_id,
                                  "name": self.find_element_by_name, "css": self.find_element_by_css_selector}

    def loop_find_element(self, func, text, timeout=10, interval=0.5):
        """
        Loop to find the target web element by func.

        Args:
            func: function to find element
            text: param of function
            timeout: time to find the element
            interval: interval between operation
        Returns:
            element that been found
        """
        start_time = time.time()
        while True:
            try:
                element = func(text)
            except NoSuchElementException:
                print("Element not found!")
                # 超时则raise，未超时则进行下次循环:
                if (time.time() - start_time) > timeout:
                    # try_log_screen(screen)
                    raise NoSuchElementException('Element %s not found in screen' % text)
                else:
                    time.sleep(interval)
            else:
                return element

    @logwrap
    def find_element_by_xpath(self, xpath):
        """
        Find the web element by xpath.

        Args:
            xpath: find the element by xpath.
        Returns:
            Web element of current page.
        """
        web_element = self.loop_find_element(super(Firefox, self).find_element_by_xpath, xpath)
        log_res = self._gen_screen_log(web_element)
        return Element(web_element, log_res)

    @logwrap
    def find_element_by_id(self, id):
        """
        Find the web element by id.

        Args:
            id: find the element by attribute id.
        Returns:
            Web element of current page.
        """
        web_element = self.loop_find_element(super(Firefox, self).find_element_by_id, id)
        log_res = self._gen_screen_log(web_element)
        return Element(web_element, log_res)

    @logwrap
    def find_element_by_css_selector(self, css_selector):
        """
        Find the web element by css_selector.

        Args:
            css_selector: find the element by attribute css_selector.
        Returns:
            Web element of current page.
        """
        web_element = self.loop_find_element(super(Firefox, self).find_element_by_css_selector, css_selector)
        log_res = self._gen_screen_log(web_element)
        return Element(web_element, log_res)

    @logwrap
    def find_element_by_name(self, name):
        """
        Find the web element by name.

        Args:
            name: find the element by attribute name.
        Returns:
            Web element of current page.
        """
        web_element = self.loop_find_element(super(Firefox, self).find_element_by_name, name)
        log_res = self._gen_screen_log(web_element)
        return Element(web_element, log_res)

    @logwrap
    def switch_to_new_tab(self):
        """
        Switch to the new tab.
        """
        _father = self.number
        self.number = len(self.window_handles) - 1
        self.father_number[self.number] = _father
        self.switch_to.window(self.window_handles[self.number])
        self._gen_screen_log()
        time.sleep(0.5)

    @logwrap
    def switch_to_previous_tab(self):
        """
        Switch to the previous tab(which to open current tab).
        """
        self.number = self.father_number[self.number]
        self.switch_to.window(self.window_handles[self.number])
        self._gen_screen_log()
        time.sleep(0.5)

    @logwrap
    def airtest_touch(self, v):
        """
        Perform the touch action on the current page by image identification.

        Args:
            v: target to touch, either a Template instance or absolute coordinates (x, y)
        Returns:
            Finial position to be clicked.
        """
        if isinstance(v, Template):
            _pos = loop_find(v, timeout=ST.FIND_TIMEOUT, driver=self)
        else:
            _pos = v
        x, y = _pos
        # self.action_chains.move_to_element_with_offset(root_element, x, y)
        # self.action_chains.click()
        pos = self._get_left_up_offset()
        pos = (pos[0] + x, pos[1] + y)
        self._move_to_pos(pos)
        self._click_current_pos()
        time.sleep(1)
        return _pos

    @logwrap
    def assert_template(self, v, msg=""):
        """
        Assert target exists on the current page.

        Args:
            v: target to touch, either a Template instance
        Raise:
            AssertionError - if target not found.
        Returns:
            Position of the template.
        """
        if isinstance(v, Template):
            try:
                pos = loop_find(v, timeout=ST.FIND_TIMEOUT, driver=self)
            except TargetNotFoundError:
                raise AssertionError("Target template not found on screen.")
            else:
                return pos
        else:
            raise IsNotTemplateError("args is not a template")

    @logwrap
    def assert_exist(self, param, operation, msg=""):
        """
        Assert element exist.

        Args:
            operation: the method that to find the element.
            param: the param of method.
        Raise:
            AssertionError - if assertion failed.
        """
        try:
            func = self.operation_to_func[operation]
        except Exception:
            raise AssertionError("There was no operation: %s" % operation)
        try:
            func(param)
        except Exception as e:
            raise AssertionError("Target element not find.")

    @logwrap
    def get(self, address):
        """
        Access the web address.

        Args:
            address: the address that to accesss
        """
        super(WebFirefox, self).get(address)
        time.sleep(2)

    @logwrap
    def back(self):
        """
        Back to last page.
        """
        super(WebFirefox, self).back()
        self._gen_screen_log()
        time.sleep(1)

    @logwrap
    def forward(self):
        """
        Forward to next page.
        """
        super(WebFirefox, self).forward()
        self._gen_screen_log()
        time.sleep(1)

    @logwrap
    def snapshot(self, filename=None):
        self._gen_screen_log(filename=filename)

    @logwrap
    def _gen_screen_log(self, element=None, filename=None, ):
        if ST.LOG_DIR is None:
            return None
        if filename:
            self.screenshot(filename)
        jpg_file_name = str(int(time.time())) + '.png'
        jpg_path = os.path.join(ST.LOG_DIR, jpg_file_name)
        print("this is jpg path:", jpg_path)
        self.screenshot(jpg_path)
        saved = {"screen": jpg_path}
        if element:
            size = element.size
            location = element.location
            x = size['width'] / 2 + location['x']
            y = size['height'] / 2 + location['y']
            if "darwin" in sys.platform:
                x, y = x * 2, y * 2
            saved.update({"pos": [[x, y]]})
        return saved

    def screenshot(self, file_path=None):
        if file_path:
            self.save_screenshot(file_path)
        else:
            if not ST.LOG_DIR:
                file_path = "temp.png"
            else:
                file_path = os.path.join(ST.LOG_DIR, "temp.png")
            self.save_screenshot(file_path)
            screen = aircv.imread(file_path)
            return screen

    def _get_left_up_offset(self):
        window_pos = self.get_window_position()
        window_size = self.get_window_size()
        mouse = Controller()
        screen = self.screenshot()
        screen_size = get_resolution(screen)
        offset = window_size["width"] - \
                 screen_size[0], window_size["height"] - screen_size[1]
        pos = (int(offset[0] / 2 + window_pos['x']),
               int(offset[1] + window_pos['y'] - offset[0] / 2))
        return pos

    def _move_to_pos(self, pos):
        self.mouse.position = pos

    def _click_current_pos(self):
        self.mouse.click(Button.left, 1)

    def to_json(self):
        # add this method for json encoder in logwrap
        return repr(self)


class Element(WebElement):

    def __init__(self, _obj, log):
        super(Element, self).__init__(
            parent=_obj._parent, id_=_obj._id, w3c=_obj._w3c)
        self.res_log = log

    def click(self):
        super(Element, self).click()
        time.sleep(0.5)
        return self.res_log

    def send_keys(self, text, keyborad=None):
        if keyborad:
            super(Element, self).send_keys(text, keyborad)
        else:
            super(Element, self).send_keys(text)
        time.sleep(0.5)
        return self.res_log



#!/usr/bin/python
# encoding=utf-8

"""
@Author  :  Lijiawei
@Date    :  8/17/2021 6:49 PM
@Desc    :  Command line.
"""

import argparse
import os
import random
import subprocess
import sys

from loguru import logger

from utx.cli.scaffold_web import init_parser_scaffold_web, main_scaffold_web
from utx.core.utils.tools import decryption, check_port, plat
from utx import __description__, __version__
from utx.cli.scaffold import init_parser_scaffold, main_scaffold, init_parser_install, main_install, \
    init_parser_uninstall, main_uninstall


def main():
    """Parse command line options and run commands.
    """
    parser = argparse.ArgumentParser(description=__description__)

    parser.add_argument(
        "-v", "--version", dest="version", action="store_true", help="show version"
    )
    subparsers = parser.add_subparsers(help="sub-command help")
    sub_parser_scaffold = init_parser_scaffold(subparsers)
    sub_parser_scaffold_web = init_parser_scaffold_web(subparsers)
    sub_parser_install = init_parser_install(subparsers)
    sub_parser_uninstall = init_parser_uninstall(subparsers)
    subparsers.add_parser(
        "test", help="debug iOS devices(UTX driver needs to be installed)."
    )
    subparsers.add_parser(
        "list", help="show connected iOS devices."
    )
    subparsers.add_parser(
        "applist", help="list packages."
    )

    if len(sys.argv) == 1:
        # utx
        parser.print_help()
        sys.exit(0)
    elif len(sys.argv) == 2:
        # print help for sub-commands
        if sys.argv[1] in ["-v", "--version"]:
            # utx -v
            print(f"{__version__}")
        elif sys.argv[1] == "test":
            # utx -test
            logger.info('UTX will connect your IOS device.')
            logger.warning(
                'Please make sure the IOS phone is connected to the computer and the UTX driver is installed!')
            devices = '{} list'.format(decryption(b'dGlkZXZpY2U='))
            d = os.popen(devices)
            if len(d.readlines()) == 0:
                logger.error('No device detected!')
                sys.exit()

            # todo 前置判断 utx driver 是否安装
            # check_driver = subprocess.Popen('{} wdaproxy'.format(decryption(b'dGlkZXZpY2U=')),
            #                                 shell=True,
            #                                 stdout=subprocess.PIPE,
            #                                 stderr=subprocess.STDOUT,
            #                                 encoding='utf-8'
            #                                 )
            # logger.info('Detecting if UTX(WDA.ipa) driver is installed.')
            #
            # check_port(port=8100)
            #
            # utx_driver = subprocess.Popen('{} ps'.format(decryption(b'dGlkZXZpY2U=')),
            #                               shell=True,
            #                               stdout=subprocess.PIPE,
            #                               stderr=subprocess.STDOUT,
            #                               encoding='utf-8').communicate()
            #
            # if 'WebDriverAgentRunner' not in utx_driver:
            #     logger.error('UTX driver(WDA.ipa) is not installed!')
            #     sys.exit()
            # else:
            #     pass

            proxy = str(random.randint(50000, 60000) + 1)
            check_port(port=proxy)
            logger.info('Debug address:http://127.0.0.1:{}'.format(proxy))
            try:
                subprocess.call('{} wdaproxy --port {}'.format(decryption(b'dGlkZXZpY2U='), proxy), shell=True)
            except KeyboardInterrupt:
                check_port(port=proxy)
                sys.exit()
            finally:
                if plat() != 'Windows':
                    os.system('clear')
                else:
                    os.system('cls')
                logger.error('')
                logger.error('-------------------- utx end --------------------------')

                logger.error('Disconnect and end debugging!')

        elif sys.argv[1] in ["-h", "--help"]:
            # utx -h
            parser.print_help()

        elif sys.argv[1] == "install":
            # utx install
            sub_parser_install.print_help()

        elif sys.argv[1] == "uninstall":
            # utx uninstall
            sub_parser_uninstall.print_help()

        elif sys.argv[1] == "list":
            # utx list
            os.system('{} list'.format(decryption(b'dGlkZXZpY2U=')))

        elif sys.argv[1] == "applist":
            # utx applist
            os.system('{} applist'.format(decryption(b'dGlkZXZpY2U=')))

        elif sys.argv[1] == "startproject":
            # utx startproject
            sub_parser_scaffold.print_help()

        elif sys.argv[1] == "startproject-web":
            # utx startproject-web
            sub_parser_scaffold_web.print_help()

        else:
            parser.print_help()
        sys.exit(0)

    args = parser.parse_args()

    if args.version:
        print(f"{__version__}")
        sys.exit(0)

    if sys.argv[1] == "startproject":
        main_scaffold(args)

    if sys.argv[1] == "startproject-web":
        main_scaffold_web(args)

    if sys.argv[1] == "install":
        main_install(args)

    if sys.argv[1] == "uninstall":
        main_uninstall(args)


def cli_env():
    parser = argparse.ArgumentParser(description='manual to this script')
    parser.add_argument("--device", type=str)
    parser.add_argument("--platform", type=str)
    parser.add_argument("--wda", type=str)
    parser.add_argument("--init", type=str)

    args = parser.parse_args()
    cli_device = args.device
    cli_platform = args.platform
    cli_wda = args.wda
    cli_init = args.init

    return cli_device, cli_platform, cli_wda, cli_init


def cli_web_env():
    parser = argparse.ArgumentParser(description='manual to this script')

    parser.add_argument("--headless", type=str)
    parser.add_argument("--driver", type=str)

    args = parser.parse_args()

    cli_headless = args.headless
    cli_driver_path = args.driver

    return cli_headless, cli_driver_path



#!/usr/bin/python
# encoding=utf-8

"""
@Author  :  Lijiawei
@Date    :  2021/12/11 11:11 上午 
@Desc    :  platform line.
"""


class TextProcessor(object):
    PLUGINS = {}

    def process(self, text, plugins=()):
        if plugins is ():
            for plugin_name in self.PLUGINS.keys():
                text = self.PLUGINS[plugin_name]().process(text)
        else:
            for plugin_name in plugins:
                text = self.PLUGINS[plugin_name]().process(text)
        return text

    @classmethod
    def plugin_register(cls, plugin_name):
        def wrapper(plugin):
            cls.PLUGINS.update({plugin_name: plugin})
            return plugin

        return wrapper



#!/usr/bin/python
# encoding=utf-8

"""
@Author  :  Lijiawei
@Date    :  9/14/2021 4:36 PM
@Desc    :  Custom Report line.
"""
import io
import os
import time

import jinja2
CUSTOM_HTML_TPL = "summary_template.html"
CUSTOM_STATIC_DIR = os.path.dirname(__file__)


def gen_report(results, report_path):
    """
    gen_report
    :param results: results list
    :param report_path: report_path
    :return: custom report html
    """
    formatList = []
    for i in results:
        if i not in formatList:
            formatList.append(i)

    for res in formatList:
        env = jinja2.Environment(
            loader=jinja2.FileSystemLoader(CUSTOM_STATIC_DIR),
            extensions=(),
            autoescape=True
        )
        template = env.get_template(CUSTOM_HTML_TPL, CUSTOM_STATIC_DIR)
        html = template.render({"results": [res]})
        now = time.strftime("%Y-%m-%d_%H-%M-%S")
        dev = res['devices']
        output_file = os.path.join(report_path, now + '_' + dev + "_summary.html")
        with io.open(output_file, 'w', encoding="utf-8") as f:
            f.write(html)
        print(output_file)


#!/usr/bin/python
# encoding=utf-8

"""
@Author  :  Lijiawei
@Date    :  11/02/2021 3:36 PM
@Desc    :  airtest_api line.
"""
import time
import os
from airtest.core.helper import G, logwrap
from airtest import aircv
from airtest.aircv import get_resolution
from airtest.core.error import TargetNotFoundError
from airtest.core.settings import Settings as ST


@logwrap
def loop_find(query, driver=None, timeout=10, threshold=None, interval=0.5, intervalfunc=None):
    """
    Search for image template in the screen until timeout

    Args:
        query: image template to be found in screenshot
        timeout: time interval how long to look for the image template
        threshold: default is None
        interval: sleep interval before next attempt to find the image template
        intervalfunc: function that is executed after unsuccessful attempt to find the image template

    Raises:
        TargetNotFoundError: when image template is not found in screenshot

    Returns:
        TargetNotFoundError if image template not found, otherwise returns the position where the image template has
        been found in screenshot

    """
    start_time = time.time()
    while True:
        screen = driver.screenshot()
        query.resolution = get_resolution(screen)
        if screen is None:
            print("Screen is None, may be locked")
        else:
            if threshold:
                query.threshold = threshold
            match_pos = query.match_in(screen)
            if match_pos:
                try_log_screen(screen)
                return match_pos

        if intervalfunc is not None:
            intervalfunc()

        # 超时则raise，未超时则进行下次循环:
        if (time.time() - start_time) > timeout:
            try_log_screen(screen)
            raise TargetNotFoundError('Picture %s not found in screen' % query)
        else:
            time.sleep(interval)


@logwrap
def try_log_screen(screen=None):
    """
    Save screenshot to file

    Args:
        screen: screenshot to be saved

    Returns:
        None

    """
    if not ST.LOG_DIR:
        return
    if screen is None:
        screen = G.DEVICE.snapshot()
    filename = "%(time)d.jpg" % {'time': time.time() * 1000}
    filepath = os.path.join(ST.LOG_DIR, filename)
    aircv.imwrite(filepath, screen)
    return filename


if __name__ == "__main__":
    from airtest.core.api import connect_device
    from airtest.core.error import AdbError, InstallMultipleError

    dev = connect_device("android://10.252.60.147:5039/AAQKYTY9MVRGOFIV")
    dev.install_multiple_app("E://windows//Test.apk")



#!/usr/bin/python
# encoding=utf-8

"""
@Author  :  Lijiawei
@Date    :  8/31/2021 9:42 PM
@Desc    :  Tools line.
"""
import base64
import platform
import subprocess
import re
from airtest.utils.logger import get_logger
from airtest.core.api import *
from airtest.core.android.adb import ADB
import os
import configparser

LOGGING = get_logger(__name__)


def allure_report(report_path, report_html):
    """
    Generate allure Report
    :param report_path:
    :param report_html:
    :return:
    """
    # 执行命令 allure generate
    allure_cmd = "allure generate %s -o %s --clean" % (report_path, report_html)
    try:
        subprocess.call(allure_cmd, shell=True)
    except Exception:
        LOGGING.error("The generation of allure report failed. Please check the relevant configuration of the test "
                      "environment")
        raise


def plat():
    """
    Check the current script running platform
    :return:'Linux', 'Windows' or 'Darwin'.
    """
    return platform.system()


def check_port(port):
    """
    Detect whether the port is occupied and clean up
    :param port:System port
    :return:None
    """
    if plat() != 'Windows':
        os.system("lsof -i:%s| grep LISTEN| awk '{print $2}'|xargs kill -9" % port)
    else:
        port_cmd = 'netstat -ano | findstr {}'.format(port)
        r = os.popen(port_cmd)
        if len(r.readlines()) == 0:
            return
        else:
            pid_list = []
            for line in r.readlines():
                line = line.strip()
                pid = re.findall(r'[1-9]\d*', line)
                pid_list.append(pid[-1])
            pid_set = list(set(pid_list))[0]
            pid_cmd = 'taskkill -PID {} -F'.format(pid_set)
            os.system(pid_cmd)


def display():
    """
    Gets the length and width of the current device
    :return:
    """
    width, height = device().get_current_resolution()
    return width, height


def dev_uri(state=None):
    """
    Perform `adb devices` command and return the list of adb devices
    :param state: optional parameter to filter devices in specific state
    :return: list od adb devices
    """
    patten = re.compile(r'^[\w\d.:-]+\t[\w]+$')
    device_list = []
    output = ADB().cmd("devices", device=False)
    for line in output.splitlines():
        line = line.strip()
        if not line or not patten.match(line):
            continue
        serialno, cstate = line.split('\t')
        if state and cstate != state:
            continue
        device_list.append(serialno)
    return device_list


def get_report(airtest_report_path):
    """
    Get the latest test report path
    :return: report name and path
    """
    file_lists = os.listdir(airtest_report_path)
    file_lists.sort(key=lambda fn: os.path.getmtime(airtest_report_path + "/" + fn)
                    if not os.path.isdir(airtest_report_path + "/" + fn) else 0)
    report = os.path.join(airtest_report_path, file_lists[-1])
    print(file_lists[-1])
    return report


def encryption(value):
    """
    encryption
    https://cdn.jsdelivr.net/gh/openutx/static/
    :param value:
    :return:
    """
    bytes_url = value.encode("utf-8")
    str_url = base64.b64encode(bytes_url)
    return str_url


def decryption(value):
    """
    decryption
    https://cdn.jsdelivr.net/gh/openutx/static/
    :param value:
    :return:
    """
    str_url = base64.b64decode(value).decode("utf-8")
    return str_url


def str_to_bool(value):
    """
    str convert bool
    :param value:
    :return:
    """
    return True if value.lower() == 'true' else False


def selector(status, flag, cases_list):
    """

    :param status:
    :param flag:
    :param cases_list:
    :return:
    """
    cases = []
    if int(status) == 0:
        for suite in cases_list:
            if flag in suite:
                if suite.endswith(".air"):
                    cases.append(suite)
    else:
        for suite in cases_list:
            if suite.endswith(".air"):
                cases.append(suite)

    return cases


class ReadConfig:
    """
    configuration file
    """

    def __init__(self, ini_path):
        self.ini_path = ini_path
        if not os.path.exists(ini_path):
            raise FileNotFoundError("Profile %s does not exist！" % ini_path)
        self.config = configparser.RawConfigParser()  # When there are% symbols, use raw to read
        self.config.read(ini_path, encoding='utf-8')

    def _get(self, section, option):
        """

        :param section:
        :param option:
        :return:
        """
        return self.config.get(section, option)

    def _set(self, section, option, value):
        """

        :param section:
        :param option:
        :param value:
        :return:
        """
        self.config.set(section, option, value)
        with open(self.ini_path, 'w') as f:
            self.config.write(f)

    def getvalue(self, env, name):
        return self._get(env, name)

    def update_value(self, env, name, value):
        return self._set(env, name, value)



#!/usr/bin/python
# encoding=utf-8

"""
@Author  :  Lijiawei
@Date    :  2021/12/11 11:22 上午 
@Desc    :  main line.
"""
from .platform import TextProcessor


def test():
    processor = TextProcessor()
    print(processor.PLUGINS)  # {’plugin1': <class '__main__.CleanMarkdownBolds'>}
    processed = processor.process(text="**foo bar**", plugins=('pg1',))
    print(processed)
    processed = processor.process(text="--foo bar--")
    print(processed)




#!/usr/bin/env python
# -*- coding:utf-8 -*-
from setuptools import setup, find_packages
from utx import __version__

with open('README.md', encoding='utf-8') as f:
    long_description = f.read()

setup(
    name="utx",
    version="{}".format(__version__),
    keywords=["utx", "airtest", "pytest", "selenium", "ui", "tools"],
    description='UTX will help you write ui automated tests more easily!',
    long_description=long_description,
    long_description_content_type='text/markdown',
    license='Apache License 2.0',

    url="https://github.com/openutx",
    author="lijiawei",
    author_email="jiawei.li2@qq.com",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console :: Curses",
        "Operating System :: MacOS",
        "Operating System :: POSIX",
        "Operating System :: Microsoft :: Windows",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: Implementation :: CPython"
    ],
    entry_points="""
    [console_scripts]
    utx = utx.cli.cli:main
    """,
    packages=find_packages(),
    include_package_data=True,
    platforms="any",
    install_requires=["airtest~=1.2.4", "tenacity~=8.0.1", "allure-pytest~=2.9.43", "pocoui~=1.0.84",
                      "tidevice~=0.5.0", "faker~=4.18.0", "jmespath~=0.10.0", "loguru~=0.5.3", "pytest-xdist~=2.3.0",
                      "PyYAML~=5.4.1", "allure-python-commons~=2.9.43", "pytest-rerunfailures~=10.1", "imageio~=2.9.0",
                      "hexdump~=3.3", "pyOpenSSL~=21.0.0", "pyasn1~=0.4.8", "selenium~=3.14.0", "pynput~=1.7.4"]
)
